@@ -12,7 +12,7 @@ WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along with
-this program.  If not, see <http://www.gnu.org/licenses/>.  */
+this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 #include "makeint.h"
 #include <string.h>
@@ -102,11 +102,11 @@ w32ify(const char *filename, int resolve)
     if (resolve)
       {
         char *fp = _fullpath (NULL, filename, sizeof (w32_path));
-        strncpy (w32_path, fp, sizeof (w32_path));
+        strncpy (w32_path, fp, sizeof (w32_path) - 1);
         free (fp);
       }
     else
-      strncpy(w32_path, filename, sizeof (w32_path));
+      strncpy(w32_path, filename, sizeof (w32_path) - 1);
 
     for (p = w32_path; p && *p; p++)
       if (*p == '\\')
