@@ -12,7 +12,7 @@ WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along with
-this program.  If not, see <http://www.gnu.org/licenses/>.  */
+this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 #include "makeint.h"
 
@@ -92,7 +92,7 @@ DWORD process_wait_for_multiple_objects(
     for (;;) {
       DWORD objectCount = nCount;
       int blockCount  = 0;
-      DWORD retVal;
+      DWORD retVal = 0;
 
       assert(bWaitAll == FALSE); /* This logic only works for this use case */
       assert(dwMilliseconds == 0 || dwMilliseconds == INFINITE); /* No support for timeouts */
